@@ -1,18 +1,31 @@
 package urpm::select;
 
-# $Id: select.pm 258637 2009-07-28 13:14:16Z cfergeau $
 
 use strict;
 use urpm::msg;
-use urpm::util;
+use urpm::util qw(any formatList intersection member min partition uniq);
 use urpm::sys;
 use URPM;
 
-my $default_priority_list = 'rpm,perl-URPM,perl-MDV-Distribconf,urpmi,meta-task,glibc,aria2';
+my $default_priority_list = 'rpm,perl-base,perl-URPM,perl-MDV-Distribconf,urpmi,meta-task,glibc,aria2';
 my @priority_list = split(',', $default_priority_list);
 
 my $evr_re = qr/[^\-]*-[^\-]*\.[^\.\-]*$/;
 
+
+=head1 NAME
+
+urpm::select - package selection routines for urpmi
+
+=head1 SYNOPSIS
+
+=head1 DESCRIPTION
+
+=over
+
+=cut
+
+
 sub add_packages_to_priority_upgrade_list {
     @priority_list = uniq(@priority_list, @_);
 }
@@ -75,15 +88,34 @@ sub build_listid_ {
     $urpm->build_listid(undef, undef, searchmedia_idlist($urpm));
 }
 
-#- search packages registered by their names by storing their ids into the $packages hash.
-#- Recognized options:
-#-	all
-#-	caseinsensitive
-#-	fuzzy
-#-      no_substring
-#-	src
-#-	use_provides
-#-
+
+=item search_packages($urpm, $packages, $names, %options)
+
+Search packages registered by their names by storing their ids into the $packages hash.
+
+Returns either 0 (error), 1 (OK) or 'substring' (fuzzy match).
+
+Recognized options:
+
+=over
+
+
+=item * all
+
+=item * caseinsensitive
+
+=item * fuzzy
+
+=item * no_substring: in --auto, do not allow to install a package substring match (you can use -a to force it)
+
+=item * src
+
+=item * use_provides
+
+=back
+
+=cut
+
 #- side-effects: $packages, flag_skip
 sub search_packages {
     my ($urpm, $packages, $names, %options) = @_;
@@ -114,17 +146,19 @@ sub _search_packages {
 	my @found;
 	$qv = '(?i)' . $qv if $options{caseinsensitive};
 
+	# First: try to find an exact match
 	if (!$options{fuzzy}) {
 	    #- try to search through provides.
-	    if (my @l = map {
+	    my @l = map {
 		    $_
 		    && ($options{src} ? $_->arch eq 'src' : $_->is_arch_compat)
 		    && ($options{use_provides} || $_->name eq $v)
 		    && defined($_->id)
 		    && (!$urpm->{searchmedia} || pkg_in_searchmedia($urpm, $_))
 		    ? $_ : @{[]};
-		} $urpm->packages_providing($v))
-	    {
+	    } $urpm->packages_providing($v);
+
+	    if (@l) {
 		$exact{$v} = _search_packages_keep_best($v, \@l, $options{all});
 		next;
 	    }
@@ -132,6 +166,7 @@ sub _search_packages {
 	    _findindeps($urpm, \%found, $qv, $v, $options{caseinsensitive}, $options{src});
 	}
 
+	# Second pass: try to find a partial match (substring) [slow]
 	foreach my $id (build_listid_($urpm)) {
 	    my $pkg = $urpm->{depslist}[$id];
 	    ($options{src} ? $pkg->arch eq 'src' : $pkg->is_arch_compat) or next;
@@ -142,14 +177,12 @@ sub _search_packages {
 	    if (!$options{fuzzy}) {
 		if ($pack eq $v) {
 		    $exact{$v} = $id;
-		    next;
 		} elsif ($pack_a eq $v) {
 		    push @{$exact_a{$v}}, $id;
-		    next;
 		} elsif ($pack_ra eq $v || $options{src} && $pack_name eq $v) {
 		    push @{$exact_ra{$v}}, $id;
-		    next;
 		}
+		next;
 	    }
 	    if ($pack =~ /$qv/) {
 		next if member($pack, @found);
@@ -186,7 +219,7 @@ sub _search_packages {
 		$urpm->{error}(N("No package named %s", $v));
 		values(%l) != 0 and $urpm->{error}(
 		    N("The following packages contain %s: %s",
-			$v, urpm::util::formatList(4, sort { $a cmp $b } keys %l)) . "\n" . 
+			$v, formatList(4, sort { $a cmp $b } keys %l)) . "\n" . 
 		    N("You should use \"-a\" to use all of them")
 		);
 		$result = 0;
@@ -238,19 +271,48 @@ sub _search_packages_keep_best {
     join('|', map { $_->id } @l);
 }
 
-#- Resolves dependencies between requested packages (and auto selection if any).
-#- handles parallel option if any.
-#- The return value is true if program should be restarted (in order to take
-#- care of important packages being upgraded (priority upgrades)
-#- %options :
-#-	auto_select
-#-	install_src
-#-	priority_upgrade
-#- %options passed to ->resolve_requested:
-#-	callback_choices
-#-	keep
-#-	nodeps
-#-	no_suggests
+
+=item resolve_dependencies($urpm, $state, $requested, %options)
+
+
+Resolves dependencies between requested packages (and auto selection if any).
+Handles parallel option if any.
+
+The return value is true if program should be restarted (in order to take
+care of important packages being upgraded (priority upgrades)
+
+$state->{selected} will contain the selection of packages to be
+installed or upgraded
+
+
+%options :
+
+=over
+
+=item * auto_select
+
+=item * install_src
+
+=item * priority_upgrade
+
+=back
+
+%options passed to ->resolve_requested:
+
+=over
+
+=item * callback_choices
+
+=item * keep
+
+=item * nodeps
+
+=item * no_recommends
+
+=back
+
+=cut
+
 sub resolve_dependencies {
     #- $state->{selected} will contain the selection of packages to be
     #- installed or upgraded
@@ -290,6 +352,7 @@ sub resolve_dependencies {
 		$need_restart = _resolve_priority_upgrades($urpm, $db, $state, $state->{selected}, \@l, %options);
 	    }
 	}
+	$urpm->{options}{'split-length'} = 0 if $need_restart;
     }
     $need_restart;
 }
@@ -299,7 +362,7 @@ sub select_replacepkgs {
 
     my $db = urpm::db_open_or_die_($urpm);
     foreach my $id (keys %$requested) {
-	my @pkgs = $urpm->find_candidate_packages_($id);
+	my @pkgs = $urpm->find_candidate_packages($id);
 	if (my ($pkg) = grep { URPM::is_package_installed($db, $_) } @pkgs) {
 		$urpm->{debug_URPM}("selecting replacepkg " . $pkg->fullname) if $urpm->{debug_URPM};
 		$pkg->set_flag_requested;
@@ -338,9 +401,9 @@ sub _resolve_priority_upgrades {
     my %priority_requested = map { $_->id => undef } @$priority_pkgs;
 
     $urpm->resolve_requested($db, \%priority_state, \%priority_requested, %options);
-    if (grep { ! exists $priority_state{selected}{$_} } keys %priority_requested) {
+    if (any { ! exists $priority_state{selected}{$_} } keys %priority_requested) {
 	#- some packages which were selected previously have not been selected, strange!
-    } elsif (grep { ! exists $priority_state{selected}{$_} } keys %$selected) {
+    } elsif (any { ! exists $priority_state{selected}{$_} } keys %$selected) {
 	#- there are other packages to install after this priority transaction.
 	%$state = %priority_state;
 	$need_restart = 1;
@@ -384,16 +447,30 @@ sub get_preferred {
     $best ? [$best] : [], [@prefer, @l];
 }
 
-my $fullname2name_re = qr/^(.*)-[^\-]*-[^\-]*\.[^\.\-]*$/;
+=item find_packages_to_remove($urpm, $state, $l, %options)
+
+Find packages to remove.
+
+Options:
+
+=over
+
+=item * callback_base
+
+=item * callback_fuzzy
+
+=item * callback_notfound
+
+=item * force
+
+=item * matches
+
+=item * test
+
+=back
+
+=cut 
 
-#- find packages to remove.
-#- options:
-#-	callback_base
-#-	callback_fuzzy
-#-	callback_notfound
-#-	force
-#-	matches
-#-	test
 sub find_packages_to_remove {
     my ($urpm, $state, $l, %options) = @_;
 
@@ -406,56 +483,20 @@ sub find_packages_to_remove {
 
 	if (!$options{matches}) {
 	    foreach (@$l) {
-		my ($n, $found);
+		my ($found);
 
-		#- check if name-version-release.architecture was given.
-		if (($n) = $_ =~ $fullname2name_re) {
-		    $db->traverse_tag('name', [ $n ], sub {
-			    my ($p) = @_;
-			    $p->fullname eq $_ or return;
-			    $urpm->resolve_rejected($db, $state, $p, removed => 1);
-			    push @m, scalar $p->fullname;
-			    $found = 1;
-			});
-		    $found and next;
-		}
-
-		#- check if name-version-release was given.
-		if (($n) = /^(.*)-[^\-]*-[^\-]*$/) {
-		    $db->traverse_tag('name', [ $n ], sub {
-			    my ($p) = @_;
-			    my ($name, $version, $release) = $p->fullname;
-			    "$name-$version-$release" eq $_ or return;
-			    $urpm->resolve_rejected($db, $state, $p, removed => 1);
-			    push @m, scalar $p->fullname;
-			    $found = 1;
-			});
-		    $found and next;
-		}
-
-		#- check if name-version was given.
-		if (($n) = /^(.*)-[^\-]*$/) {
-		    $db->traverse_tag('name', [ $n ], sub {
-			    my ($p) = @_;
-			    my ($name, $version) = $p->fullname;
-			    "$name-$version" eq $_ or return;
-			    $urpm->resolve_rejected($db, $state, $p, removed => 1);
-			    push @m, scalar $p->fullname;
-			    $found = 1;
-			});
-		    $found and next;
-		}
-
-		#- check if only name was given.
-		$db->traverse_tag('name', [ $_ ], sub {
+		$db->traverse_tag('nvra', [ $_ ], sub {
 			my ($p) = @_;
 			$urpm->resolve_rejected($db, $state, $p, removed => 1);
 			push @m, scalar $p->fullname;
 			$found = 1;
 		    });
-		$found and next;
 
-		push @notfound, $_;
+		if ($found) {
+		    next;
+		} else {
+		    push @notfound, $_;
+		}
 	    }
 	    if (!$options{force} && @notfound && @$l > 1) {
 		$options{callback_notfound} && $options{callback_notfound}->($urpm, @notfound)
@@ -500,7 +541,7 @@ sub find_packages_to_remove {
 	find_removed_from_basesystem($urpm, $db, $state, $options{callback_base})
 	    or return ();
     }
-    removed_packages($urpm, $state);
+    removed_packages($state);
 }
 
 sub find_removed_from_basesystem {
@@ -517,7 +558,7 @@ sub find_removed_from_basesystem {
 sub _prohibit_packages_that_would_be_removed {
     my ($urpm, $db, $state) = @_;
 
-    my @to_remove = removed_packages($urpm, $state) or return 1;
+    my @to_remove = removed_packages($state) or return 1;
 
     my @dont_remove = ('basesystem', 'basesystem-minimal', 
 		       split /,\s*/, $urpm->{global_config}{'prohibit-remove'});
@@ -529,17 +570,28 @@ sub _prohibit_packages_that_would_be_removed {
     });
 
     grep {
-	! grep { $base{$_} } rejected_unsatisfied($state, $_);
+	! any { $base{$_} } rejected_unsatisfied($state, $_);
     } intersection(\@to_remove, \@base_fn);
 }
 
-#- misc functions to help finding ask_unselect and ask_remove elements with their reasons translated.
+
+=item unselected_packages($state)
+
+misc functions to help finding ask_unselect and ask_remove elements with their reasons translated.
+
+=cut
+
 sub unselected_packages {
-    my (undef, $state) = @_;
+    my ($state) = @_;
     grep { $state->{rejected}{$_}{backtrack} } keys %{$state->{rejected} || {}};
 }
 
-#- misc functions to help finding ask_unselect and ask_remove elements with their reasons translated.
+=item already_installed($state)
+
+misc functions to help finding ask_unselect and ask_remove elements with their reasons translated.
+
+=cut
+
 sub already_installed {
     my ($state) = @_;
     uniq(map { scalar $_->fullname } values %{$state->{rejected_already_installed} || {}});
@@ -598,7 +650,7 @@ sub was_pkg_name_installed {
 }
 
 sub removed_packages {
-    my (undef, $state) = @_;
+    my ($state) = @_;
     grep {
 	$state->{rejected}{$_}{removed} && !$state->{rejected}{$_}{obsoleted};
     } keys %{$state->{rejected} || {}};
@@ -628,7 +680,7 @@ sub conflicting_packages_msg {
 
 sub removed_packages_msgs {
     my ($urpm, $state) = @_;
-    map { translate_why_removed_one($urpm, $state, $_) } sort(removed_packages($urpm, $state));
+    map { translate_why_removed_one($urpm, $state, $_) } sort(removed_packages($state));
 }
 
 sub translate_why_removed {
@@ -641,7 +693,7 @@ sub translate_why_removed_one {
     my $closure = rejected_closure($state, $fullname) or return $fullname;
 
     my ($from) = keys %$closure;
-    my ($whyk) = keys %{$closure->{$from}};
+    my ($whyk) = sort { $b ne 'avoid' } keys %{$closure->{$from}};
     my $whyv = $closure->{$from}{$whyk};
     my $frompkg = $urpm->search($from, strict_fullname => 1);
     my $s = do {
@@ -666,7 +718,7 @@ sub translate_why_removed_one {
 }
 
 sub _libdb_version { $_[0] =~ /libdb-(\S+)\.so/ ? version->new("v$1") : () }
-sub _rpm_version() { `rpm --version` =~ /version ([0-9.]+)$/ ? version->new("v$1") : () }
+sub _rpm_version() { `rpm --version` =~ /version ([0-9.]+)(?:-(beta|rc).*)?$/ ? version->new("v$1") : () }
 
 sub should_we_migrate_back_rpmdb_db_version {
     my ($urpm, $state) = @_;
@@ -702,3 +754,7 @@ sub should_we_migrate_back_rpmdb_db_version {
 }
 
 1;
+
+=back
+
+=cut
